@@ -1,6 +1,8 @@
 mod common;
 
+use bytes::{Buf, BufMut, BytesMut};
 use tokio::time::sleep;
+use tokio_util::codec::{Decoder, Encoder};
 use tracing::*;
 use tracing_subscriber::filter::LevelFilter;
 
@@ -24,7 +26,7 @@ impl Pea2Pea for JoJoNode {
 impl Handshake for JoJoNode {
     async fn perform_handshake(&self, conn: Connection) -> io::Result<Connection> {
         // some handshakes are useful, others are menacing ゴゴゴゴ
-        match !conn.side {
+        match !conn.side() {
             ConnectionSide::Initiator => {
                 info!(parent: self.node().span(), "Dio!");
                 sleep(Duration::from_secs(4)).await;
@@ -59,20 +61,36 @@ impl From<u8> for BattleCry {
     }
 }
 
+struct SingleByteCodec;
+
+impl Decoder for SingleByteCodec {
+    type Item = BattleCry;
+    type Error = io::Error;
+
+    fn decode(&mut self, src: &mut BytesMut) -> Result<Option<Self::Item>, Self::Error> {
+        if src.is_empty() {
+            return Ok(None);
+        }
+        Ok(Some(src.get_u8().into()))
+    }
+}
+
+impl Encoder<BattleCry> for SingleByteCodec {
+    type Error = io::Error;
+
+    fn encode(&mut self, item: BattleCry, dst: &mut BytesMut) -> Result<(), Self::Error> {
+        dst.put_u8(item as u8);
+        Ok(())
+    }
+}
+
 #[async_trait::async_trait]
 impl Reading for JoJoNode {
     type Message = BattleCry;
+    type Codec = SingleByteCodec;
 
-    fn read_message<R: io::Read>(
-        &self,
-        _source: SocketAddr,
-        reader: &mut R,
-    ) -> io::Result<Option<Self::Message>> {
-        let mut arr = [0u8; 1];
-        reader.read_exact(&mut arr)?;
-        let battle_cry = BattleCry::from(arr[0]);
-
-        Ok(Some(battle_cry))
+    fn codec(&self, _addr: SocketAddr) -> Self::Codec {
+        SingleByteCodec
     }
 
     async fn process_message(
@@ -85,6 +103,12 @@ impl Reading for JoJoNode {
             BattleCry::Muda => BattleCry::Ora,
         };
 
+        if reply == BattleCry::Ora {
+            info!(parent: self.node().span(), "{:?}!", reply);
+        } else {
+            warn!(parent: self.node().span(), "{:?}!", reply);
+        };
+
         self.send_direct_message(source, reply)
             .unwrap()
             .await
@@ -96,22 +120,10 @@ impl Reading for JoJoNode {
 
 impl Writing for JoJoNode {
     type Message = BattleCry;
+    type Codec = SingleByteCodec;
 
-    fn write_message<W: io::Write>(
-        &self,
-        _: SocketAddr,
-        payload: &Self::Message,
-        writer: &mut W,
-    ) -> io::Result<()> {
-        writer.write_all(&[*payload as u8])?;
-
-        if *payload == BattleCry::Ora {
-            info!(parent: self.node().span(), "{:?}!", payload);
-        } else {
-            warn!(parent: self.node().span(), "{:?}!", payload);
-        };
-
-        Ok(())
+    fn codec(&self, _addr: SocketAddr) -> Self::Codec {
+        SingleByteCodec
     }
 }
 
