@@ -47,16 +47,10 @@ impl Handshake for NakedNode {
 #[async_trait::async_trait]
 impl Reading for NakedNode {
     type Message = String;
+    type Codec = common::TestCodec<Self::Message>;
 
-    fn read_message<R: io::Read>(
-        &self,
-        _source: SocketAddr,
-        reader: &mut R,
-    ) -> io::Result<Option<Self::Message>> {
-        let vec = common::read_len_prefixed_message::<R, 2>(reader)?;
-
-        vec.map(|v| String::from_utf8(v).map_err(|_| io::ErrorKind::InvalidData.into()))
-            .transpose()
+    fn codec(&self, _addr: SocketAddr) -> Self::Codec {
+        Default::default()
     }
 
     async fn process_message(&self, source: SocketAddr, message: Self::Message) -> io::Result<()> {
@@ -85,15 +79,10 @@ impl Reading for NakedNode {
 
 impl Writing for NakedNode {
     type Message = String;
+    type Codec = common::TestCodec<Self::Message>;
 
-    fn write_message<W: io::Write>(
-        &self,
-        _: SocketAddr,
-        payload: &Self::Message,
-        buffer: &mut W,
-    ) -> io::Result<()> {
-        buffer.write_all(&(payload.len() as u16).to_le_bytes())?;
-        buffer.write_all(payload.as_bytes())
+    fn codec(&self, _addr: SocketAddr) -> Self::Codec {
+        Default::default()
     }
 }
 
