@@ -1,5 +1,6 @@
 mod common;
 
+use bytes::BytesMut;
 use once_cell::sync::Lazy;
 use parking_lot::Mutex;
 use rand::{rngs::SmallRng, seq::IteratorRandom, Rng, SeedableRng};
@@ -8,6 +9,7 @@ use tokio::{
     io::{AsyncReadExt, AsyncWriteExt},
     time::sleep,
 };
+use tokio_util::codec::{Decoder, Encoder};
 use tracing::*;
 use tracing_subscriber::filter::LevelFilter;
 
@@ -86,32 +88,35 @@ impl Handshake for Player {
     async fn perform_handshake(&self, mut conn: Connection) -> io::Result<Connection> {
         let mut buffer = [0u8; 16];
 
-        let peer_name = match !conn.side {
+        let node_conn_side = !conn.side();
+        let stream = self.borrow_stream(&mut conn);
+
+        let peer_name = match node_conn_side {
             ConnectionSide::Initiator => {
                 // send own PlayerName
                 let own_name = self.node().name().as_bytes().to_vec();
-                conn.writer().write_all(&own_name).await?;
+                stream.write_all(&own_name).await?;
 
                 // receive the peer's PlayerName
-                let len = conn.reader().read(&mut buffer).await?;
+                let len = stream.read(&mut buffer).await?;
 
                 String::from_utf8_lossy(&buffer[..len]).into_owned()
             }
             ConnectionSide::Responder => {
                 // receive the peer's PlayerName
-                let len = conn.reader().read(&mut buffer).await?;
+                let len = stream.read(&mut buffer).await?;
                 let peer_name = String::from_utf8_lossy(&buffer[..len]).into_owned();
 
                 // send own PlayerName
                 let own_name = self.node().name().as_bytes().to_vec();
-                conn.writer().write_all(&own_name).await?;
+                stream.write_all(&own_name).await?;
 
                 peer_name
             }
         };
 
         let player = PlayerInfo {
-            addr: conn.addr,
+            addr: conn.addr(),
             is_carrier: false,
         };
         self.other_players.lock().insert(peer_name, player);
@@ -126,20 +131,25 @@ enum Message {
     IHaveThePotato(PlayerName),
 }
 
+impl Decoder for common::TestCodec<Message> {
+    type Item = Message;
+    type Error = io::Error;
+
+    fn decode(&mut self, src: &mut BytesMut) -> Result<Option<Self::Item>, Self::Error> {
+        self.0
+            .decode(src)?
+            .map(|b| bincode::deserialize(&b).map_err(|_| io::ErrorKind::InvalidData.into()))
+            .transpose()
+    }
+}
+
 #[async_trait::async_trait]
 impl Reading for Player {
     type Message = Message;
+    type Codec = common::TestCodec<Self::Message>;
 
-    fn read_message<R: io::Read>(
-        &self,
-        _source: SocketAddr,
-        reader: &mut R,
-    ) -> io::Result<Option<Self::Message>> {
-        // expecting inbound messages to be prefixed with their length encoded as a LE u16
-        let vec = common::read_len_prefixed_message::<R, 2>(reader)?;
-
-        vec.map(|v| bincode::deserialize(&v).map_err(|_| io::ErrorKind::InvalidData.into()))
-            .transpose()
+    fn codec(&self, _addr: SocketAddr) -> Self::Codec {
+        Default::default()
     }
 
     async fn process_message(&self, _source: SocketAddr, message: Self::Message) -> io::Result<()> {
@@ -174,19 +184,21 @@ impl Reading for Player {
     }
 }
 
+impl<M> Encoder<Message> for common::TestCodec<M> {
+    type Error = io::Error;
+
+    fn encode(&mut self, item: Message, dst: &mut BytesMut) -> Result<(), Self::Error> {
+        let bytes = bincode::serialize(&item).unwrap().into();
+        self.0.encode(bytes, dst)
+    }
+}
+
 impl Writing for Player {
     type Message = Message;
+    type Codec = common::TestCodec<Self::Message>;
 
-    fn write_message<W: io::Write>(
-        &self,
-        _: SocketAddr,
-        payload: &Self::Message,
-        writer: &mut W,
-    ) -> io::Result<()> {
-        let payload_len = bincode::serialized_size(payload).unwrap() as u16;
-        writer.write_all(&payload_len.to_le_bytes())?;
-        bincode::serialize_into(writer, payload)
-            .map_err(|e| io::Error::new(io::ErrorKind::Other, e))
+    fn codec(&self, _addr: SocketAddr) -> Self::Codec {
+        Default::default()
     }
 }
 
