@@ -10,11 +10,7 @@ use pea2pea::{
     Node, Pea2Pea, Topology,
 };
 
-use std::{
-    io::{self, Read},
-    net::SocketAddr,
-    time::Duration,
-};
+use std::{io, net::SocketAddr, time::Duration};
 
 #[derive(Clone)]
 struct Player(Node);
@@ -30,33 +26,10 @@ const NUM_PLAYERS: usize = 100;
 #[async_trait::async_trait]
 impl Reading for Player {
     type Message = String;
+    type Codec = common::TestCodec<Self::Message>;
 
-    fn read_message<R: io::Read>(
-        &self,
-        _src: SocketAddr,
-        reader: &mut R,
-    ) -> io::Result<Option<String>> {
-        let mut len_arr = [0u8; 2];
-        if reader.read_exact(&mut len_arr).is_err() {
-            return Ok(None);
-        }
-        let payload_len = u16::from_le_bytes(len_arr) as usize;
-
-        if payload_len == 0 {
-            return Err(io::ErrorKind::InvalidData.into());
-        }
-
-        let mut buffer = vec![0u8; payload_len];
-        if reader
-            .take(payload_len as u64)
-            .read_exact(&mut buffer)
-            .is_err()
-        {
-            Ok(None)
-        } else {
-            let str = String::from_utf8(buffer).map_err(|_| io::ErrorKind::InvalidData)?;
-            Ok(Some(str))
-        }
+    fn codec(&self, _addr: SocketAddr) -> Self::Codec {
+        Default::default()
     }
 
     async fn process_message(&self, source: SocketAddr, message: String) -> io::Result<()> {
@@ -83,15 +56,10 @@ impl Reading for Player {
 
 impl Writing for Player {
     type Message = String;
+    type Codec = common::TestCodec<Self::Message>;
 
-    fn write_message<W: io::Write>(
-        &self,
-        _: SocketAddr,
-        payload: &Self::Message,
-        writer: &mut W,
-    ) -> io::Result<()> {
-        writer.write_all(&(payload.len() as u16).to_le_bytes())?;
-        writer.write_all(payload.as_bytes())
+    fn codec(&self, _addr: SocketAddr) -> Self::Codec {
+        Default::default()
     }
 }
 
